@@ -29,8 +29,6 @@
 #include "main.h"
 #include "dialog.h"
 
-static const __WCHAR helpfileW[] = { 'n','o','t','e','p','a','d','.','h','l','p',0 };
-
 static INT_PTR WINAPI DIALOG_AboutLilyPadDlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
 static INT_PTR WINAPI DIALOG_PAGESETUP_DlgProc(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
 
@@ -138,6 +136,10 @@ static VOID DoSaveFile(VOID)
     DWORD dwNumWrite;
     LPSTR pTemp;
     DWORD size;
+#ifdef UNICODE
+    LPWSTR pwTemp;
+    DWORD wsize;
+#endif
 
     hFile = CreateFile(Globals.szFileName, GENERIC_WRITE, FILE_SHARE_WRITE,
                        NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
@@ -147,6 +149,30 @@ static VOID DoSaveFile(VOID)
         return;
     }
 
+#ifdef UNICODE
+    wsize = GetWindowTextLengthW(Globals.hEdit) + 1;
+    pwTemp = HeapAlloc(GetProcessHeap(), 0, wsize * sizeof(WCHAR));
+    if (!pwTemp)
+    {
+	CloseHandle(hFile);
+        ShowLastError();
+        return;
+    }
+    wsize = GetWindowTextW(Globals.hEdit, pwTemp, wsize) + 1;
+
+    size = WideCharToMultiByte(CP_UTF8, 0, pwTemp, wsize ,
+			       NULL, 0, NULL, NULL);
+    pTemp = HeapAlloc(GetProcessHeap(), 0, size);
+    if (!pTemp)
+    {
+	HeapFree(GetProcessHeap(), 0, pwTemp);
+	CloseHandle(hFile);
+        ShowLastError();
+        return;
+    }
+    size = WideCharToMultiByte(CP_UTF8, 0, pwTemp, wsize,
+			       pTemp, size, NULL, NULL) - 1;
+#else
     size = GetWindowTextLengthA(Globals.hEdit) + 1;
     pTemp = HeapAlloc(GetProcessHeap(), 0, size);
     if (!pTemp)
@@ -156,6 +182,7 @@ static VOID DoSaveFile(VOID)
         return;
     }
     size = GetWindowTextA(Globals.hEdit, pTemp, size);
+#endif
 
     if (!WriteFile(hFile, pTemp, size, &dwNumWrite, NULL))
         ShowLastError();
@@ -164,6 +191,9 @@ static VOID DoSaveFile(VOID)
 
     SetEndOfFile(hFile);
     CloseHandle(hFile);
+#ifdef UNICODE
+    HeapFree(GetProcessHeap(), 0, pwTemp);
+#endif
     HeapFree(GetProcessHeap(), 0, pTemp);
 }
 
@@ -204,10 +234,16 @@ void DoOpenFile(__LPCWSTR szFileName)
 {
     static const __WCHAR dotlog[] = { '.','L','O','G',0 };
     HANDLE hFile;
-    __LPWSTR pTemp;
+    LPSTR pTemp;
     DWORD size;
     DWORD dwNumRead;
     __WCHAR log[5];
+    int counter;
+    int i;
+#ifdef UNICODE
+    LPWSTR pwTemp;
+    DWORD wsize;
+#endif
 
     /* Close any files and prompt to save changes */
     if (!DoCloseFile())
@@ -228,9 +264,8 @@ void DoOpenFile(__LPCWSTR szFileName)
 	ShowLastError();
 	return;
     }
-    size++;
 
-    pTemp = HeapAlloc(GetProcessHeap(), 0, size);
+    pTemp = HeapAlloc(GetProcessHeap(), 0, size + 1);
     if (!pTemp)
     {
 	CloseHandle(hFile);
@@ -249,17 +284,88 @@ void DoOpenFile(__LPCWSTR szFileName)
     CloseHandle(hFile);
     pTemp[dwNumRead] = 0;
 
-#ifdef UNICODE
-    if (IsTextUnicode(pTemp, dwNumRead, NULL))
+    /* newline normalization */
+    counter = 0;
+    for ( i=0; i<dwNumRead; i++ )
     {
-	__LPWSTR p = (__LPWSTR)pTemp;
-	/* We need to strip BOM Unicode character, SetWindowTextW won't do it for us. */
-	if (*p == 0xFEFF || *p == 0xFFFE) p++;
-	SetWindowText(Globals.hEdit, p);
+	switch (pTemp[i])
+	{
+	    case '\r':
+	        if (pTemp[i+1]=='\n')
+	            i++;
+	        else
+		    counter++;
+	        break;
+	    case '\n':
+	        counter++;
+	        break;
+	    case 0:
+	        dwNumRead = i;
+	        break;
+	}
     }
-    else
+    if (counter)
+    {
+	LPSTR p;
+	DWORD dwNewSize = dwNumRead + counter;
+
+	p = HeapAlloc(GetProcessHeap(), 0, dwNewSize + 1);
+	if (p)
+	{
+	    int i_in = 0;
+	    int i_out = 0;
+
+	    while ((i_in < dwNumRead) && (i_out < dwNewSize))
+	    {
+		char c=pTemp[i_in];
+		switch (c)
+		{
+		    case '\r':
+		        p[i_out++] = '\r';
+		        p[i_out] = '\n';
+		        if (pTemp[i_in+1]=='\n')
+			    i_in++;
+		        break;
+		    case '\n':
+		        p[i_out++] = '\r';
+		        p[i_out] = '\n';
+		        break;
+		    case 0:
+		        p[i_out] = 0;
+		        dwNumRead = i_in;
+		        dwNewSize = i_out;
+		        break;
+		    default:
+		        p[i_out] = c;
+		        break;
+		}
+		i_in++;
+		i_out++;
+	    }
+
+	    p[dwNewSize] = 0;
+	    HeapFree(GetProcessHeap(), 0, pTemp);
+	    pTemp = p;
+	    dwNumRead = dwNewSize;
+	}
+    }
+
+#ifdef UNICODE
+    wsize = MultiByteToWideChar(CP_UTF8, 0, pTemp, dwNumRead + 1, NULL, 0);
+    pwTemp = HeapAlloc(GetProcessHeap(), 0, wsize * sizeof(WCHAR));
+    if (!pwTemp)
+    {
+	HeapFree(GetProcessHeap(), 0, pTemp);
+	ShowLastError();
+	return;
+    }
+    MultiByteToWideChar(CP_UTF8, 0, pTemp, dwNumRead + 1, pwTemp, wsize);
+    if (*pwTemp == 0xFEFF) pwTemp++;
+    SetWindowText(Globals.hEdit, pwTemp);
+    HeapFree(GetProcessHeap(), 0, pwTemp);
+#else
+    SetWindowText(Globals.hEdit, pTemp);
 #endif
-	SetWindowText(Globals.hEdit, pTemp);
 
     HeapFree(GetProcessHeap(), 0, pTemp);
 
@@ -666,21 +772,6 @@ VOID DIALOG_SearchNext(VOID)
         LILYPAD_DoFind(&Globals.lastFind);
 }
 
-VOID DIALOG_HelpContents(VOID)
-{
-    WinHelp(Globals.hMainWnd, helpfileW, HELP_INDEX, 0);
-}
-
-VOID DIALOG_HelpSearch(VOID)
-{
-        /* Search Help */
-}
-
-VOID DIALOG_HelpHelp(VOID)
-{
-    WinHelp(Globals.hMainWnd, helpfileW, HELP_HELPONHELP, 0);
-}
-
 VOID DIALOG_HelpAboutLilyPad(VOID)
 {
   DialogBox (Globals.hInstance, MAKEINTRESOURCE(DIALOG_ABOUTLILYPAD),
@@ -751,15 +842,6 @@ static INT_PTR WINAPI DIALOG_PAGESETUP_DlgProc(HWND hDlg, UINT msg, WPARAM wPara
           EndDialog(hDlg, IDCANCEL);
           return TRUE;
 
-        case IDHELP:
-        {
-          /* FIXME: Bring this to work */
-          static const __WCHAR sorryW[] = { 'S','o','r','r','y',',',' ','n','o',' ','h','e','l','p',' ','a','v','a','i','l','a','b','l','e',0 };
-          static const __WCHAR helpW[] = { 'H','e','l','p',0 };
-          MessageBox(Globals.hMainWnd, sorryW, helpW, MB_ICONEXCLAMATION);
-          return TRUE;
-        }
-
 	default:
 	    break;
         }
